==> libmatrix-0.03.tbz: libmatrix-0.03/ivector.c <==
/*
ivector.c

matrix manipulation library
functions on int vectors
version 0.03
Peter A. Cejchan, GPL
Mon Mar 29 12:59:02 GMT+1 2004 <cej@gli.cas.cz | pac7@post.cz | pac@plan9.cz>


*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "ivector.h" 
#include "imatrix.h" 


// functions:

int 
ivec_read(iVector *V)
// read whitespace-delimited vector
{
	Biobuf bin;
	char *buf, *av[MAXLENGTH];
	int i, j;

	Binit(&bin, 0, OREAD);
	i=0;
	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		V->len=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<V->len; j++){
			V->x[j]=atoi(av[j]);
		}
		free(buf);
	}
	return 1;
}

int
ivec_neighborpair(iVector *V, iMatrix *M)
// matrix predecessor/successor combinations
{
	int i, j;
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			M->x[i][j]=0;
		}
	}
	for(i=0; i<V->len-1; i++){
		M->x[V->x[i]][V->x[i+1]]=1;
		M->x[V->x[i+1]][V->x[i]]=1;
	}

}

iVector *
ivec_alloc(int len){
// allocates memory for an int vector

	int i;
	iVector *V;

	V=(iVector*)malloc(sizeof(iVector));
	V->len=len; 
	V->x=(int*)malloc(V->len*sizeof(int*));
	for(i=0;i<V->len;i++)
		V->x[i]=0.0;
	return V;
}

void 
ivec_write(iVector *V)
// write out a vector
{
	int i;
	print("%d ", V->len);
	for(i=0; i<V->len; i++){
		print("%d ", V->x[i]);
	}
	print("\n");
	return;
}


int 
ivec_max(iVector *V)
//prints out col index of first occurrence of max value
{
	int i, maxi;
	int maxx;
	maxx = -MAXINT;
	for(i=0; i<V->len; i++){
		if(V->x[i] > maxx) {
			maxx = V->x[i];
			maxi = i;
		}
	}
	print("%d\n", maxi);
	return 1;
}

int 
ivec_scalar_prod(iVector *v1,iVector *v2)
// SUM[ v1i*v2i ]
{
  int i;
  int sum = 0;

  for(i=0; i<v1->len; i++)
     sum += v1->x[i]*v2->x[i];
  return sum;
}

==> libmatrix-0.04.tbz: libmatrix-0.04/ivector.c <==
/*
ivector.c

matrix manipulation library
version 0.04
Peter A. Cejchan, GPL
Mon Jul 31 12:24:25 MET 2006
<cej@gli.cas.cz | pac7@post.cz>

int and bool  vector structure and functions

*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "ivector.h" 
#include "imatrix.h" 


// functions:

int 
ivec_read(iVector *V)
// read whitespace-delimited vector
{
	char *buf, *av[MAXLENGTH];
	int  j;

	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		V->len=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<V->len; j++){
			V->x[j]=atoi(av[j]);
		}
		free(buf);
	}
	return 1;
}

int
ivec_neighborpair(iVector *V, iMatrix *M)
// matrix predecessor/successor combinations
{
	int i, j;
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			M->x[i][j]=0;
		}
	}
	for(i=0; i<V->len-1; i++){
		M->x[V->x[i]][V->x[i+1]]=1;
		M->x[V->x[i+1]][V->x[i]]=1;
	}
	return 1;
}

iVector *
ivec_alloc(int len){
// allocates memory for an int vector

	int i;
	iVector *V;

	V=(iVector*)malloc(sizeof(iVector));
	V->len=len; 
	V->x=(int*)malloc(V->len*sizeof(int));
	for(i=0;i<V->len;i++)
		V->x[i]=0.0;
	return V;
}

void 
ivec_write(iVector *V)
// write out a vector
{
	int i;
	print("%d ", V->len);
	for(i=0; i<V->len; i++){
		print("%d ", V->x[i]);
	}
	print("\n");
	return;
}


int 
ivec_wheremax(iVector *V)
//prints out col index of first occurrence of max value
{
	int i, maxi=0;
	int maxx;
	maxx = -MAXINT;
	for(i=0; i<V->len; i++){
		if(V->x[i] > maxx) {
			maxx = V->x[i];
			maxi = i;
		}
	}
	print("%d\n", maxi);
	return 1;
}

int 
ivec_max(iVector *V)
//prints out  max value found
{
	int i, maxx=0;
	maxx = -MAXINT;
	for(i=0; i<V->len; i++){
		if(V->x[i] > maxx) {
			maxx = V->x[i];
		}
	}
	print("%d\n", maxx);
	return 1;
}

int 
ivec_scalar_prod(iVector *v1,iVector *v2)
// SUM[ v1i*v2i ]
{
  int i;
  int sum = 0;

  for(i=0; i<v1->len; i++)
     sum += v1->x[i]*v2->x[i];
  return sum;
}
